#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2004-2005 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
gen_sip_api.py generates .api files for eric3 using sip

Usage: Simply call it with a mode option and the output path.

        python gen_sip_api.py mode-option /path/to/api-files
"""

import sys, os, string

VERSION = "20060123"

def usage(rcode = 2):
    """Display a usage message and exit.

    rcode is the return code passed back to the calling process.
    """
    global progName

    print "Usage:"
    print "    %s option apifiledir" % (progName)
    print
    print "where:"
    print "    apifiledir    pathname of directory receiving the generated api files"
    print
    print "option (one of):"
    print "    -h,  --help                   show this help message"
    print "    -v,  --version                show the version information"
    print "    -q,  --qt    or --pyqt        generate api files for PyQt"
    print "    -k,  --kde   or --pykde       generate api files for PyKDE"
    print "    -w,  --qwt   or --pyqwt       generate api files for PyQwt"
    print "    -3,  --qwt3d or --pyqwt3d     generate api files for PyQwt3D"
    print "    -q4, --qt4   or --pyqt4       generate api files for PyQt4"
    
    sys.exit(rcode)
    
def version(rcode = 3):
    global progName
    
    print "%s Version %s" % (progName, VERSION)
    
    sys.exit(rcode)
    
def findinpath(file):
    if os.path.isabs(file):
        if os.access(file, os.X_OK):
            return file
        else:
            return None
        
    path = os.getenv('PATH')
    
    # environment variable not defined
    if path is None:
        return None
        
    dirs = string.split(path, os.pathsep)
    for dir in dirs:
        nfile = os.path.join(dir, file)
        if os.access(nfile, os.X_OK):
            return nfile
            
    return None
    
progName = os.path.basename(sys.argv[0])

if len(sys.argv) == 1 or \
   sys.argv[1] not in ["-v", "--version",
                       "-h", "--help",
                       "-q", "--qt", "--pyqt",
                       "-k", "--kde", "--pykde",
                       "-w", "--qwt", "--pyqwt",
                       "-3", "--qwt3d", "--pyqwt3d",
                       "-q4", "--qt4", "--pyqt4",
                      ]:
    usage(1)

if sys.argv[1] in ["-v", "--version"]:
    version()
elif sys.argv[1] in ["-h", "--help"]:
    usage(1)
    
if len(sys.argv) < 3:
    usage(1)
    
if sys.argv[1] in ["-q", "--qt", "--pyqt"]:
    mode = "qt"
elif sys.argv[1] in ["-k", "--kde", "--pykde"]:
    mode = "kde"
elif sys.argv[1] in ["-w", "--qwt", "--pyqwt"]:
    mode = "qwt"
elif sys.argv[1] in ["-3", "--qwt3d", "--pyqwt3d"]:
    mode = "qwt3d"
elif sys.argv[1] in ["-q4","--qt4","--pyqt4"]:
    mode = "qt4"
    
apidir = sys.argv[2]

if sys.platform == "win32":
    plattag = "WS_WIN"
    sip = "sip.exe"
elif sys.platform == "darwin":
    plattag = "WS_MACX"
    sip = "sip"
else:
    plattag = "WS_X11"
    sip = "sip"
    


sip = findinpath(sip)
if sip is None:
    print "Could not find the sip executable in your search path. Aborting..."
    sys.exit(3)
    
if not os.path.isdir(apidir):
    print "Generating the api directory."
    os.makedirs(apidir)
    
apidir = os.path.abspath(apidir)

os.chdir(apidir)

if mode in ['qt','kde','qwt','qwt3d']:

    from qt import qVersion as qtVersion
    # Qt-snapshots return a qVersion() string longer than 5 chars
    qversion = qtVersion()[:5]
    # some Qt-versions have no tag
    qttag = { 
        '3.0.0' : 'Qt_3_0_0',
        '3.0.1' : 'Qt_3_0_1',
        '3.0.2' : 'Qt_3_0_2',
        '3.0.3' : 'Qt_3_0_2',
        '3.0.4' : 'Qt_3_0_4',
        '3.0.5' : 'Qt_3_0_5',
        '3.0.6' : 'Qt_3_0_6',
        '3.0.7' : 'Qt_3_0_6',
        '3.1.0' : 'Qt_3_1_0',
        '3.1.1' : 'Qt_3_1_1',
        '3.1.2' : 'Qt_3_1_2',
        '3.2.0' : 'Qt_3_2_0',
        '3.2.1' : 'Qt_3_2_0',
        '3.2.2' : 'Qt_3_2_0',
        '3.2.3' : 'Qt_3_2_0',
        '3.3.0' : 'Qt_3_3_0',
        '3.3.1' : 'Qt_3_3_0',
        '3.3.2' : 'Qt_3_3_0',
        '3.3.3' : 'Qt_3_3_0',
        '3.3.4' : 'Qt_3_3_0',
        }.get(qversion)
    if qttag is None:
        qttag = 'Qt_3_3_0'

    if mode == "qt":
        import pyqtconfig
        
        sipdir = pyqtconfig._pkg_config['pyqt_sip_dir']
        sipdir = os.path.abspath(sipdir)
        if not os.path.isdir(sipdir):
            print "The directory (%s) containing the sipfiles does not exist. Aborting..." % sipdir
            sys.exit(2)
        
        modules = pyqtconfig._pkg_config['pyqt_modules'].split()
        for mod in modules:
            try:
                for sipfile in os.listdir(os.path.join(sipdir, mod)):
                    if sipfile.endswith('mod.sip'):
                        apifile = os.path.join(apidir, sipfile.replace('mod.sip', '.api'))
                        if sipfile.endswith('qtpemod.sip'):
                            continue
                        elif sipfile.endswith('qtextmod.sip'):
                            try:
                                from qtext import QSCINTILLA_VERSION
                                major = QSCINTILLA_VERSION >> 16
                                minor = (QSCINTILLA_VERSION >> 8) & 0xff
                                qstag = "QScintilla_%d_%d" % (major, minor)
                                args = [sip, '-a', apifile, '-t', plattag, '-t', qttag, 
                                        '-t', qstag, '-I', os.path.join(sipdir, mod), 
                                        '-I', sipdir, sipfile]
                            except:
                                continue
                        else:
                            args = [sip, '-a', apifile, '-t', plattag, '-t', qttag, 
                                    '-I', os.path.join(sipdir, mod), '-I', sipdir, sipfile]
                        print "Generating %s ..." % apifile
                        ret = os.spawnv(os.P_WAIT, sip, args)
                        if ret != 0:
                            print "Error: the process returned the exit code %d" % ret
            except OSError:
                print "Warning: The module '%s' does not exist." % mod
    
    elif mode == "kde":
        import pykdeconfig
        
        sipdir = pykdeconfig._pkg_config['pykde_sip_dir']
        sipdir = os.path.abspath(sipdir)
        if not os.path.isdir(sipdir):
            print "The directory (%s) containing the sipfiles does not exist. Aborting..." % sipdir
            sys.exit(2)
        
        from kdecore import KDE
        
        kversion = KDE.versionString()[:5]
        # some KDE-versions have no tag
        kdetag = {
            '3.0.0' : "KDE_3_0_0",
            '3.0.1' : "KDE_3_0_1",
            '3.0.2' : "KDE_3_0_1",
            '3.0.3' : "KDE_3_0_3",
            '3.1.0' : "KDE_3_1_0",
            '3.1.1' : "KDE_3_1_1",
            '3.1.2' : "KDE_3_1_2",
            '3.1.3' : "KDE_3_1_3",
            '3.1.4' : "KDE_3_1_4",
            '3.1.5' : "KDE_3_1_5",
            '3.2.0' : "KDE_3_2_0",
            '3.2.1' : "KDE_3_2_1",
            '3.2.2' : "KDE_3_2_2",
            '3.2.3' : "KDE_3_2_3",
            '3.3.0' : "KDE_3_3_0",
            '3.3.1' : "KDE_3_3_1",
            '3.3.2' : "KDE_3_3_2",
            '3.4.0' : "KDE_3_4_0",
        }.get(kversion)
        if kdetag is None:
            kdetag = 'KDE_3_3_0'
        
        pykde_imports = {
            "dcop":     [],
            "kdecore":  ["dcop"],
            "kdesu":    ["dcop", "kdecore"],
            "kdefx":    ["dcop", "kdecore"],
            "kdeui":    ["dcop", "kdecore", "kdefx"],
            "kio":      ["dcop", "kdecore", "kdefx", "kdeui"],
            "kutils":   ["dcop", "kdecore", "kdefx", "kdeui", "kio"],
            "kfile":    ["dcop", "kdecore", "kdefx", "kdeui", "kio"],
            "kparts":   ["dcop", "kdecore", "kdefx", "kdeui", "kio"],
            "khtml":    ["dcop", "kdecore", "kdefx", "kdeui", "kutils", "kio", "kparts"],
            "kjs":      ["dcop", "kdecore", "kdefx", "kdeui", "kio", "kparts", "khtml"],
            "kspell":   ["dcop", "kdecore", "kdefx", "kdeui", "kio", "kfile"],
            "kdeprint": ["dcop", "kdecore", "kdefx", "kdeui"],
            "kmdi":     ["dcop", "kdecore", "kdefx", "kdeui", "kio", "kparts"],
            "kspell2":  ["dcop", "kdecore", "kdefx", "kdeui"],
        }
        
        modules = pykdeconfig._pkg_config['pykde_modules'].split()
        for mod in modules:
            try:
                for sipfile in os.listdir(os.path.join(sipdir, mod)):
                    if sipfile.endswith('mod.sip'):
                        apifile = os.path.join(apidir, sipfile.replace('mod.sip', '.api'))
                        args = [sip, '-a', apifile, '-t', plattag, '-t', qttag, '-t', kdetag,
                                '-I', os.path.join(sipdir, mod), '-I', sipdir]
                        imports = pykde_imports.get(mod)
                        if imports:
                            for imp in imports:
                                args.extend(['-I', os.path.join(sipdir, imp)])
                        args.append(sipfile)
                        print "Generating %s ..." % apifile
                        ret = os.spawnv(os.P_WAIT, sip, args)
                        if ret != 0:
                            print "Error: the process returned the exit code %d" % ret
            except OSError:
                print "Warning: The module '%s' does not exist." % mod
    
    elif mode == "qwt":
        try:
            import pyqwtconfig
        except ImportError:
            try:
                import qwt.pyqwtconfig as pyqwtconfig
            except ImportError:
                # this is a hack until pyqwt has a proper config file
                import pyqtconfig as pyqwtconfig
                pyqwtconfig._pkg_config['pyqwt_sip_dir'] = \
                    pyqwtconfig._pkg_config['pyqt_sip_dir']
                pyqwtconfig._pkg_config['pyqwt_modules'] = 'qwt iqt'
    
        sipdir = pyqwtconfig._pkg_config['pyqwt_sip_dir']
        sipdir = os.path.abspath(sipdir)
        if not os.path.isdir(sipdir):
            print "The directory (%s) containing the sipfiles does not exist. Aborting..." % sipdir
            sys.exit(2)
        
        modules = pyqwtconfig._pkg_config['pyqwt_modules'].split()
        for mod in modules:
            try:
                for sipfile in os.listdir(os.path.join(sipdir, mod)):
                    if sipfile.endswith('mod.sip'):
                        apifile = os.path.join(apidir, sipfile.replace('mod.sip', '.api'))
                        args = [sip, '-a', apifile, '-t', plattag, '-t', qttag, 
                                '-I', os.path.join(sipdir, mod), '-I', sipdir, sipfile]
                        print "Generating %s ..." % apifile
                        ret = os.spawnv(os.P_WAIT, sip, args)
                        if ret != 0:
                            print "Error: the process returned the exit code %d" % ret
            except OSError:
                print "Warning: The module '%s' does not exist." % mod
    
    elif mode == "qwt3d":
        try:
            import pyqwt3dconfig
        except ImportError:
            try:
                import Qwt3D.pyqwt3dconfig as pyqwt3dconfig
            except ImportError:
                # this is a hack until pyqwt has a proper config file
                import pyqtconfig as pyqwt3dconfig
                pyqwt3dconfig._pkg_config['pyqwt3d_sip_dir'] = \
                    pyqwt3dconfig._pkg_config['pyqt_sip_dir']
                pyqwt3dconfig._pkg_config['pyqwt3d_modules'] = 'Qwt3D'
        import sipconfig
    
        siptag = "SIP_%s" % sipconfig._pkg_config['sip_version_str']
        siptag = siptag.replace(".", "_")
        
        sipdir = pyqwt3dconfig._pkg_config['pyqwt3d_sip_dir']
        sipdir = os.path.abspath(sipdir)
        if not os.path.isdir(sipdir):
            print "The directory (%s) containing the sipfiles does not exist. Aborting..." % sipdir
            sys.exit(2)
        
        modules = pyqwt3dconfig._pkg_config['pyqwt3d_modules'].split()
        for mod in modules:
            try:
                for sipfile in os.listdir(os.path.join(sipdir, mod)):
                    if sipfile.endswith('mod.sip'):
                        apifile = os.path.join(apidir, sipfile.replace('mod.sip', '.api'))
                        args = [sip, '-a', apifile, '-t', plattag, '-t', qttag, '-t', siptag,
                                '-I', os.path.join(sipdir, mod), '-I', sipdir, sipfile]
                        print "Generating %s ..." % apifile
                        ret = os.spawnv(os.P_WAIT, sip, args)
                        if ret != 0:
                            print "Error: the process returned the exit code %d" % ret
            except OSError:
                print "Warning: The module '%s' does not exist." % mod

elif mode == "qt4":
    import PyQt4.pyqtconfig
    from PyQt4.QtCore import qVersion as qt4Version
    
    q4version = qt4Version()
    #Qt 4.0.x is not supported by PyQt4
    qt4tag = { 
        '4.1.0' : 'Qt_4_1_0',
        '4.1.1' : 'Qt_4_1_0',
        '4.1.2' : 'Qt_4_1_0',
        }.get(q4version)
    if qt4tag is None:
        qt4tag = 'Qt_4_1_0'

    sipdir = PyQt4.pyqtconfig._pkg_config['pyqt_sip_dir']
    sipdir = os.path.abspath(sipdir)
    if not os.path.isdir(sipdir):
        print "The directory (%s) containing the sipfiles does not exist. Aborting..." % sipdir
        sys.exit(2)
    modules = PyQt4.pyqtconfig._pkg_config['pyqt_modules'].split()
    for mod in modules:
        try:
            for sipfile in os.listdir(os.path.join(sipdir, mod)):
                if sipfile.endswith('mod.sip'):
                    apifile = os.path.join(apidir, sipfile.replace('mod.sip', '.api'))
                    args = [sip, '-a', apifile, '-t', plattag, '-t', qt4tag, 
                             '-I', os.path.join(sipdir, mod), '-I', sipdir, sipfile]
                    print "Generating %s ..." % apifile
                    ret = os.spawnv(os.P_WAIT, sip, args)
                    if ret != 0:
                        print "Error: the process returned the exit code %d" % ret
        except OSError:
            print "Warning: The module '%s' does not exist." % mod

print "Done."
